#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import pkg_resources

# import interfaces
from pyams_skin.interfaces import IContextTitlePrefix
from pyams_skin.interfaces.configuration import IStaticConfiguration, IConfiguration, SKIN_CONFIGURATION_KEY, \
    IBackOfficeConfiguration, SKIN_BACK_CONFIGURATION_KEY
from pyams_utils.interfaces.site import IStaticConfigurationManager, IConfigurationManager, IConfigurationFactory, \
    IBackOfficeConfigurationFactory
from pyams_utils.interfaces.tales import ITALESExtension
from zope.annotation.interfaces import IAnnotations
from zope.traversing.interfaces import ITraversable

# import packages
from persistent import Persistent
from pyams_file.property import FileProperty
from pyams_file.views.file import FileView
from pyams_utils.adapter import ContextRequestViewAdapter, ContextAdapter, adapter_config
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyramid.httpexceptions import HTTPNotFound
from pyramid.threadlocal import get_current_registry
from pyramid.view import view_config
from zope.container.contained import Contained
from zope.interface import implementer, Interface
from zope.location.location import locate
from zope.schema.fieldproperty import FieldProperty

from pyams_skin import _


#
# Static configuration
#

@adapter_config(context=(IStaticConfigurationManager, Interface, Interface), provides=IStaticConfiguration)
class StaticConfiguration(ContextRequestViewAdapter):
    """Default static configuration"""

    application_package = 'pyams_skin'
    application_name = 'PyAMS'
    inner_package = None
    inner_package_name = None

    version_location = 'menus'

    include_header = True
    include_top_links = True
    include_site_search = True
    site_search_placeholder = _("Search...")
    site_search_handler = '#search.html'
    include_mobile_search = True
    mobile_search_placeholder = _("Search...")
    mobile_search_handler = '#search.html'
    include_user_activity = True
    include_user_shortcuts = True
    include_logout_button = True
    include_minify_button = True
    include_flags = False
    include_menus = True
    include_ribbon = True
    include_reload_button = True
    body_css_class = 'fixed-header fixed-ribbon fixed-navigation'

    @property
    def version(self):
        result = pkg_resources.get_distribution(self.application_package).version
        if self.inner_package:
            result += ' (%s v%s)' % (self.inner_package_name,
                                     pkg_resources.get_distribution(self.inner_package).version)
        return result


@adapter_config(name='static_configuration', context=(Interface, Interface, Interface), provides=ITALESExtension)
class StaticConfigurationTalesExtension(ContextRequestViewAdapter):
    """extension:static_configuration TALES expression"""

    def render(self, context=None):
        if context is None:
            context = self.context
        target = get_parent(context, IStaticConfigurationManager)
        registry = self.request.registry
        return registry.queryMultiAdapter((target, self.request, self.view), IStaticConfiguration)


#
# Dynamic front-office configuration
# May be overriden by custom applications
#

@implementer(IConfiguration)
class Configuration(Persistent, Contained):
    """Manageable configuration"""

    title = FieldProperty(IConfiguration['title'])
    short_title = FieldProperty(IConfiguration['short_title'])
    description = FieldProperty(IConfiguration['description'])
    author = FieldProperty(IConfiguration['author'])
    icon = FileProperty(IConfiguration['icon'])
    logo = FileProperty(IConfiguration['logo'])

    @staticmethod
    def get_title_prefix(request):
        adapter = get_current_registry().queryMultiAdapter((request.context, request), IContextTitlePrefix)
        if adapter is not None:
            return adapter.prefix


@adapter_config(context=IConfigurationManager, provides=IConfiguration)
def ConfigurationFactory(context):
    """Configuration factory"""
    annotations = IAnnotations(context)
    configuration = annotations.get(SKIN_CONFIGURATION_KEY)
    if configuration is None:
        registry = check_request().registry
        factory = registry.queryAdapter(context, IConfigurationFactory, default=Configuration)
        configuration = annotations[SKIN_CONFIGURATION_KEY] = factory()
        locate(configuration, context, '++configuration++')
    return configuration


@adapter_config(name='configuration', context=(Interface, Interface, Interface), provides=ITALESExtension)
class ConfigurationTalesExtension(ContextRequestViewAdapter):
    """extension:configuration TALES expression"""

    def render(self, context=None):
        if context is None:
            context = self.context
        manager = get_parent(context, IConfigurationManager)
        if manager is not None:
            return IConfiguration(manager)


@adapter_config(name='configuration', context=IConfigurationManager, provides=ITraversable)
class ConfigurationTraverser(ContextAdapter):
    """++configuration++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        return IConfiguration(self.context)


@view_config(name='favicon.ico', context=IConfigurationManager)
def site_icon(request):
    configuration = IConfiguration(request.context)
    if configuration.icon is not None:
        request = request.copy()
        request.context = configuration.icon
        return FileView(request)
    return HTTPNotFound()


#
# Dynamic back-office configuration
# May be overriden by custom applications
#

@implementer(IBackOfficeConfiguration)
class BackOfficeConfiguration(Persistent, Contained):
    """Back-office manageable configuration"""

    title = FieldProperty(IBackOfficeConfiguration['title'])
    short_title = FieldProperty(IBackOfficeConfiguration['short_title'])
    login_header = FieldProperty(IBackOfficeConfiguration['login_header'])
    login_footer = FieldProperty(IBackOfficeConfiguration['login_footer'])
    icon = FileProperty(IBackOfficeConfiguration['icon'])
    logo = FileProperty(IBackOfficeConfiguration['logo'])
    display_content_icon = FieldProperty(IBackOfficeConfiguration['display_content_icon'])
    display_shared_tool_title = FieldProperty(IBackOfficeConfiguration['display_shared_tool_title'])

    @staticmethod
    def get_title_prefix(request):
        adapter = get_current_registry().queryMultiAdapter((request.context, request), IContextTitlePrefix)
        if adapter is not None:
            return adapter.prefix


@adapter_config(context=IConfigurationManager, provides=IBackOfficeConfiguration)
def BackOfficeConfigurationFactory(context):
    """Back-office configuration factory"""
    annotations = IAnnotations(context)
    configuration = annotations.get(SKIN_BACK_CONFIGURATION_KEY)
    if configuration is None:
        registry = check_request().registry
        factory = registry.queryAdapter(context, IBackOfficeConfigurationFactory, default=BackOfficeConfiguration)
        configuration = annotations[SKIN_BACK_CONFIGURATION_KEY] = factory()
        locate(configuration, context, '++back-configuration++')
    return configuration


@adapter_config(name='back_configuration', context=(Interface, Interface, Interface), provides=ITALESExtension)
class BackOfficeConfigurationTalesExtension(ContextRequestViewAdapter):
    """extension:back_configuration TALES expression"""

    def render(self, context=None):
        if context is None:
            context = self.context
        manager = get_parent(context, IConfigurationManager)
        if manager is not None:
            return IBackOfficeConfiguration(manager)


@adapter_config(name='back-configuration', context=IConfigurationManager, provides=ITraversable)
class BackOfficeConfigurationTraverser(ContextAdapter):
    """++back-configuration++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        return IBackOfficeConfiguration(self.context)


@view_config(name='back-favicon.ico', context=IConfigurationManager)
def back_office_site_icon(request):
    configuration = IBackOfficeConfiguration(request.context)
    if configuration.icon is not None:
        request = request.copy()
        request.context = configuration.icon
        return FileView(request)
    return HTTPNotFound()
